from plone.app.multilingual import api
from plone.app.multilingual.browser.utils import multilingualMoveObject
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.multilingual.testing import PAM_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.base.interfaces import ILanguage
from plone.dexterity.utils import createContentInContainer
from plone.i18n.interfaces import ILanguageSchema
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from zope.component import getUtility
from zope.interface import alsoProvides

import transaction
import unittest


class PAMFuncTestHelperViews(unittest.TestCase):
    layer = PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization", f"Basic {TEST_USER_NAME}:{TEST_USER_PASSWORD}"
        )
        self.settings = getUtility(IRegistry).forInterface(
            ILanguageSchema, prefix="plone"
        )

    def test_universal_link_view(self):
        self.settings.use_request_negotiation = True
        self.browser.addHeader("Accept-Language", "ca")

        a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )
        a_en = api.translate(a_ca, "en")
        api.translate(a_ca, "es")

        transaction.commit()

        self.browser.open(a_en.absolute_url())
        self.browser.getLink("Universal link").click()
        self.assertEqual(self.browser.url, a_ca.absolute_url())


class PAMIntTestHelperViews(unittest.TestCase):
    layer = PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)

    def test_move_content_proper_language_folder(self):
        f_ca = createContentInContainer(
            self.portal["ca"], "Folder", title="Test folder"
        )

        a_ca = createContentInContainer(
            self.portal["ca"]["test-folder"], "Document", title="Test document"
        )

        # Change the content language of the created folder to 'es'
        multilingualMoveObject(f_ca, "es")

        self.assertIn(f_ca.id, self.portal["es"].objectIds())
        self.assertEqual(f_ca, self.portal["es"][f_ca.id])

        self.assertIn(a_ca.id, self.portal["es"][f_ca.id].objectIds())
        self.assertEqual(a_ca, self.portal["es"][f_ca.id][a_ca.id])

        adapter = ILanguage(self.portal["es"][f_ca.id])
        self.assertEqual(adapter.get_language(), "es")

        adapter = ILanguage(self.portal["es"][f_ca.id][a_ca.id])
        self.assertEqual(adapter.get_language(), "es")

    def test_move_content_existing_translation_inside(self):
        f_ca = createContentInContainer(
            self.portal["ca"], "Folder", title="Test folder"
        )

        a_ca = createContentInContainer(
            self.portal["ca"]["test-folder"], "Document", title="Test document"
        )

        a_en = api.translate(a_ca, "en")
        translations = ITranslationManager(self.portal["en"][a_en.id])
        self.assertEqual(
            translations.get_translations(),
            {
                "ca": self.portal["ca"][f_ca.id][a_ca.id],
                "en": self.portal["en"][a_ca.id],
            },
        )

        self.assertIn(a_en.id, self.portal["en"].objectIds())
        self.assertEqual(a_en, self.portal["en"][a_en.id])

        # Change the content language of the created folder to 'en'
        multilingualMoveObject(f_ca, "en")

        self.assertIn(f_ca.id, self.portal["en"].objectIds())
        self.assertEqual(f_ca, self.portal["en"][f_ca.id])

        self.assertIn(a_ca.id, self.portal["en"][f_ca.id].objectIds())
        self.assertEqual(a_ca, self.portal["en"][f_ca.id][a_ca.id])

        adapter = ILanguage(self.portal["en"][f_ca.id])
        self.assertEqual(adapter.get_language(), "en")

        adapter = ILanguage(self.portal["en"][f_ca.id][a_ca.id])
        self.assertEqual(adapter.get_language(), "en")

        translations = ITranslationManager(self.portal["en"][f_ca.id][a_ca.id])
        self.assertEqual(
            translations.get_translations(), {"en": self.portal["en"][f_ca.id][a_ca.id]}
        )

        translations = ITranslationManager(self.portal["en"][a_en.id])
        self.assertEqual(
            translations.get_translations(), {"en": self.portal["en"][a_en.id]}
        )

    def test_modify_translations_delete(self):
        createContentInContainer(self.portal["ca"], "Folder", title="Test folder")

        a_ca = createContentInContainer(
            self.portal["ca"]["test-folder"], "Document", title="Test document"
        )

        a_en = api.translate(a_ca, "en")

        view = a_en.restrictedTraverse("modify_translations")()
        self.assertIn(
            'href="http://nohost/plone/ca/test-folder/test-document/delete_confirmation" '  # noqa§
            'title="Delete translated item"',
            view,
            "modify_translations was missing delete link for translation",
        )

        # Test https://github.com/plone/plone.app.multilingual/pull/283
        self.assertNotIn(
            'href="http://nohost/plone/en/test-document/delete_confirmation" '  # noqa§
            'title="Delete translated item"',
            view,
            "modify_translations contained delete link for the context",
        )



from email.header import Header
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.robotframework import AutoLogin
from plone.app.robotframework import Content
from plone.app.robotframework import RemoteLibraryLayer
from plone.app.robotframework.remote import RemoteLibrary
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from plone.dexterity.utils import iterSchemata
from plone.dexterity.utils import iterSchemataForType
from plone.rfc822 import constructMessageFromSchemata
from plone.rfc822 import initializeObjectFromSchemata
from plone.testing import z2
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.configuration import xmlconfig
from zope.event import notify
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from zope.lifecycleevent import ObjectModifiedEvent

import plone.app.dexterity
import plone.app.multilingual


try:
    from plone.app.robotframework.utils import disableCSRFProtection
except ImportError:

    def disableCSRFProtection():
        pass


class PloneAppMultilingualLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Configure ZCML
        xmlconfig.file(
            "testing.zcml", plone.app.multilingual, context=configurationContext
        )

        xmlconfig.file(
            "overrides.zcml", plone.app.multilingual, context=configurationContext
        )

        # Enable languageindependent-field on IRelatedItems-behavior
        from plone.app.relationfield.behavior import IRelatedItems

        alsoProvides(IRelatedItems["relatedItems"], ILanguageIndependentField)

    def setUpPloneSite(self, portal):
        # Activate product
        applyProfile(portal, "plone.app.multilingual:default")

        # Empower test user
        setRoles(portal, TEST_USER_ID, ["Manager"])

        # Enable all errors
        error_log = getToolByName(portal, "error_log")
        error_log._ignored_exceptions = ()

        # Set default workflow
        wftool = getToolByName(portal, "portal_workflow")
        wftool.setDefaultChain("simple_publication_workflow")

        # Cleanup p.a.contenttypes stuff
        if "robot-test-folder" in portal.objectIds():
            portal.manage_delObjects("robot-test-folder")


PLONE_APP_MULTILINGUAL_FIXTURE = PloneAppMultilingualLayer()

PLONE_APP_MULTILINGUAL_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_MULTILINGUAL_FIXTURE,), name="plone.app.multilingual:Integration"
)


class PloneAppMultiLingualPresetLayer(PloneSandboxLayer):
    """Test installation with preset languages."""

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Configure ZCML
        xmlconfig.file(
            "testing.zcml", plone.app.multilingual, context=configurationContext
        )

        xmlconfig.file(
            "overrides.zcml", plone.app.multilingual, context=configurationContext
        )

        # Enable languageindependent-field on IRelatedItems-behavior
        from plone.app.relationfield.behavior import IRelatedItems

        alsoProvides(IRelatedItems["relatedItems"], ILanguageIndependentField)

    def setUpPloneSite(self, portal):
        # Define available languages before installing PAM
        # This simulates the behavior of having predefined languages
        # in GenericSetup before installing PAM
        language_tool = getToolByName(portal, "portal_languages")
        language_tool.addSupportedLanguage("ca")
        language_tool.addSupportedLanguage("es")

        # Enable request negotiator
        language_tool.use_request_negotiation = True

        # Activate product
        applyProfile(portal, "plone.app.multilingual:default")

        # Empower test user
        setRoles(portal, TEST_USER_ID, ["Manager"])

        # Enable all errors
        error_log = getToolByName(portal, "error_log")
        error_log._ignored_exceptions = ()

        # Set default workflow
        wftool = getToolByName(portal, "portal_workflow")
        wftool.setDefaultChain("simple_publication_workflow")

        # Cleanup p.a.contenttypes stuff
        if "robot-test-folder" in portal.objectIds():
            portal.manage_delObjects("robot-test-folder")

        # PLEASE DELETE ME!
        # I am a ugly hack to work around
        # https://github.com/plone/Products.CMFPlone/issues/2589
        # until it is solved properly.
        from plone.app.multilingual.dx.schemaeditor import get_li_schema
        from plone.schemaeditor.interfaces import IFieldEditorExtender
        from zope.component import provideAdapter

        provideAdapter(
            get_li_schema,
            provides=IFieldEditorExtender,
            name="plone.schemaeditor.languageindependent",
        )


PLONE_APP_MULTILINGUAL_PRESET_FIXTURE = PloneAppMultiLingualPresetLayer()

PLONE_APP_MULTILINGUAL_PRESET_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_MULTILINGUAL_PRESET_FIXTURE,),
    name="plone.app.multilingual:PresetIntegration",
)


class MultipleLanguagesLayer(z2.Layer):
    defaultBases = (PLONE_APP_MULTILINGUAL_PRESET_FIXTURE,)


MULTIPLE_LANGUAGES_LAYER = MultipleLanguagesLayer()

PLONE_APP_MULTILINGUAL_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(MULTIPLE_LANGUAGES_LAYER,), name="plone.app.multilingual:Functional"
)


class MultiLingual(RemoteLibrary):
    def create_content_type(self, portal_type):
        """Create dummy content type with a single custom field"""
        disableCSRFProtection()
        fti = DexterityFTI(str(portal_type), title=portal_type)
        fti.behaviors = (
            "plone.basic",
            "plone.translatable",
        )
        fti.model_source = """\
<model xmlns="http://namespaces.plone.org/supermodel/schema">
<schema>
<field name="custom" type="zope.schema.TextLine">
  <description />
  <required>False</required>
  <title>Custom field</title>
</field>
</schema>
</model>"""
        fti.global_allow = True
        self.portal_types._setObject(str(portal_type), fti)

        # TODO: This should be split into two separate keywords:
        # - Create content type
        # - Add content type field

    def set_field_language_independent(self, portal_type, field, value="1"):
        """Set the given field in the given portal type language independent
        or unset from being one
        """
        disableCSRFProtection()
        for schema in iterSchemataForType(portal_type):
            if field in schema:
                ob = schema[field]
                if ILanguageIndependentField.providedBy(ob):
                    if value.lower() not in ("true", "on", "yes", "y", "1"):
                        noLongerProvides(schema[ob], ILanguageIndependentField)
                else:
                    if value.lower() in ("true", "on", "yes", "y", "1"):
                        alsoProvides(ob, ILanguageIndependentField)

    def create_translation(self, *args, **kwargs):
        """Create translation for an object with uid in the given
        target_language and return its UID

        Usage::

            Create translation  /plone/en/foo  ca  title=Translated
        """
        disableCSRFProtection()
        # Parse arguments:
        uid_or_path = args[0]
        target_language = args[1]

        # BBB: Support keywords arguments with robotframework < 2.8.3
        kwargs.update(dict([arg.split("=", 1) for arg in args[2:]]))

        # Look up translatable content
        pc = getToolByName(self, "portal_catalog")
        uid_results = pc.unrestrictedSearchResults(UID=uid_or_path)
        path_results = pc.unrestrictedSearchResults(
            path={"query": uid_or_path.rstrip("/"), "depth": 0}
        )
        obj = (uid_results or path_results)[0]._unrestrictedGetObject()

        # Translate
        manager = ITranslationManager(obj)
        manager.add_translation(target_language)
        translation = manager.get_translation(target_language)

        # Update fields
        data = constructMessageFromSchemata(obj, iterSchemata(obj))
        for key, value in kwargs.items():
            del data[key]
            data[key] = Header(value, "utf-8")
        del data["language"]
        initializeObjectFromSchemata(translation, iterSchemata(obj), data)
        notify(ObjectModifiedEvent(translation))

        # Return uid for the translation
        return IUUID(translation)


REMOTE_LIBRARY_BUNDLE_FIXTURE = RemoteLibraryLayer(
    bases=(PLONE_FIXTURE,),
    libraries=(AutoLogin, Content, MultiLingual),
    name="RemoteLibraryBundle:RobotRemote",
)

PLONE_APP_MULTILINGUAL_ROBOT_TESTING = FunctionalTesting(
    bases=(MULTIPLE_LANGUAGES_LAYER, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="plone.app.multilingual:Robot",
)


PAM_INTEGRATION_TESTING = PLONE_APP_MULTILINGUAL_INTEGRATION_TESTING
PAM_INTEGRATION_PRESET_TESTING = PLONE_APP_MULTILINGUAL_PRESET_INTEGRATION_TESTING
PAM_FUNCTIONAL_TESTING = PLONE_APP_MULTILINGUAL_FUNCTIONAL_TESTING
PAM_ROBOT_TESTING = PLONE_APP_MULTILINGUAL_ROBOT_TESTING



from Acquisition import aq_parent
from plone.app.multilingual.interfaces import ILanguageIndependentFieldsManager
from plone.app.multilingual.interfaces import ILanguageRootFolder
from plone.app.multilingual.interfaces import ITranslationCloner
from plone.app.multilingual.interfaces import ITranslationFactory
from plone.app.multilingual.interfaces import ITranslationIdChooser
from plone.app.multilingual.interfaces import ITranslationLocator
from plone.app.multilingual.interfaces import ITranslationManager
from plone.base.interfaces.siteroot import IPloneSiteRoot
from zope.interface import implementer


@implementer(ILanguageIndependentFieldsManager)
class DefaultLanguageIndependentFieldsManager:
    """Default language independent fields manager."""

    def __init__(self, context):
        self.context = context

    def get_field_names(self):
        return []

    def copy_fields(self, translation):
        return


@implementer(ITranslationLocator)
class DefaultTranslationLocator:
    def __init__(self, context):
        self.context = context

    def __call__(self, language):
        """
        Look for the closest translated folder or siteroot
        """
        parent = aq_parent(self.context)
        translated_parent = parent
        found = False
        while (
            not (
                IPloneSiteRoot.providedBy(parent)
                and not ILanguageRootFolder.providedBy(parent)
            )
            and not found
        ):
            parent_translation = ITranslationManager(parent)
            if parent_translation.has_translation(language):
                translated_parent = parent_translation.get_translation(language)
                found = True
            parent = aq_parent(parent)
        return translated_parent


@implementer(ITranslationCloner)
class DefaultTranslationCloner:
    def __init__(self, context):
        self.context = context

    def __call__(self, obj):
        return


@implementer(ITranslationIdChooser)
class DefaultTranslationIdChooser:
    def __init__(self, context):
        self.context = context

    def __call__(self, parent, language):
        content_id = self.context.getId()
        splitted = content_id.split("-")
        # ugly heuristic (searching for something like 'de', 'en' etc.)
        if len(splitted) > 1 and len(splitted[-1]) == 2:
            content_id = "-".join(splitted[:-1])
        while content_id in parent.objectIds():
            content_id = f"{content_id}-{language}"
        return content_id


@implementer(ITranslationFactory)
class DefaultTranslationFactory:
    def __init__(self, context):
        self.context = context

    def __call__(self, language):
        content_type = self.context.portal_type
        # parent for translation
        locator = ITranslationLocator(self.context)
        parent = locator(language)
        # id for translation
        name_chooser = ITranslationIdChooser(self.context)
        content_id = name_chooser(parent, language)
        # creating the translation
        new_id = parent.invokeFactory(
            type_name=content_type, id=content_id, language=language
        )
        new_content = getattr(parent, new_id)
        # clone language-independent content
        cloner = ITranslationCloner(self.context)
        cloner(new_content)
        return new_content



from .interfaces import ILanguageIndependentField
from Acquisition import aq_base
from plone.app.multilingual.manager import TranslationManager
from Products.CMFCore.utils import getToolByName
from z3c.form.interfaces import DISPLAY_MODE
from z3c.form.interfaces import IValue
from z3c.form.interfaces import NO_VALUE
from z3c.form.validator import StrictSimpleFieldValidator
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.pagetemplate.interfaces import IPageTemplate


def isLanguageIndependent(field):
    return field.interface is not None and ILanguageIndependentField.providedBy(field)


class LanguageIndependentFieldValidator(StrictSimpleFieldValidator):
    """Override validator so we can ignore language independent fields,
    these will be automatically filled later on by subscriber.createdEvent
    """

    def validate(self, value, force=False):
        # always pass
        pass


class LanguageIndependentFieldInputTemplate:
    """Override input template for language independent fields with
    display widget, because values will be automatically filled
    by later on by subscriber.createdEvent.
    """

    def __init__(self, context, request, view, field, widget):
        self.context = context
        self.request = request
        self.view = view
        self.field = field
        self.widget = widget

    def __call__(self, widget):
        template = getMultiAdapter(
            (
                self.context,
                self.request,
                self.view,
                self.field,
                self.widget,
            ),
            IPageTemplate,
            name=DISPLAY_MODE,
        )
        return template(widget)


@implementer(IValue)
class ValueBase:
    def __init__(self, context, request, form, field, widget):
        self.context = context
        self.request = request
        self.field = field
        self.form = form
        self.widget = widget

    @property
    def catalog(self):
        return getToolByName(self.context, "portal_catalog")


class AddingLanguageIndependentValue(ValueBase):
    # XXX Deprecated ???
    def getTranslationUuid(self):
        translation_info = getattr(self.request, "translation_info", {})
        if "tg" in translation_info.keys():
            return translation_info["tg"]

    def get(self):
        uuid = self.getTranslationUuid()

        if isLanguageIndependent(self.field) and uuid:
            manager = TranslationManager(uuid)
            result = manager.get_translations()

            if len(result) >= 1:
                orig_lang = list(result.keys())[0]
                obj = result[orig_lang]
                name = self.field.__name__
                # XXX
                # this does not work with behaviors, if other than direct
                # attribute storage was used.
                try:
                    value = getattr(aq_base(obj), name)
                except AttributeError:
                    pass
                else:
                    return value

        if self.field.default is None:
            return NO_VALUE

        return self.field.default



from Acquisition import aq_parent
from plone.app.multilingual import _
from plone.app.multilingual.browser.vocabularies import untranslated_languages
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.base.interfaces import IPloneSiteRoot
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import interface
from zope import schema
from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IBrowserSubMenuItem
from zope.component.hooks import getSite
from zope.interface import provider
from zope.schema.interfaces import IContextAwareDefaultFactory


def make_relation_root_path(context):
    ctx = getSite()
    if not IPloneSiteRoot.providedBy(ctx):
        ctx = aq_parent(ctx)
    return "/".join(ctx.getPhysicalPath())


class IMultilingualLayer(interface.Interface):
    """browser layer"""


class ITranslateSubMenuItem(IBrowserSubMenuItem):
    """The menu item linking to the translate menu."""


class ITranslateMenu(IBrowserMenu):
    """The translate menu."""


class ICreateTranslation(interface.Interface):
    language = schema.Choice(
        title=_("title_language", default="Language"),
        source=untranslated_languages,
    )


class IUpdateLanguage(interface.Interface):
    language = schema.Choice(
        title=_("title_available_languages", default="Available languages"),
        description=_(
            "description_update_language",
            default="Untranslated languages from the current content",
        ),
        source=untranslated_languages,
        required=True,
    )


@provider(IContextAwareDefaultFactory)
def request_language(context):
    return context.REQUEST.form.get("language")


class IConnectTranslation(model.Schema):
    language = schema.Choice(
        title=_("title_language", default="Language"),
        source=untranslated_languages,
        defaultFactory=request_language,
        required=True,
    )
    content = RelationChoice(
        title=_("content"),
        vocabulary="plone.app.multilingual.RootCatalog",
        required=True,
    )
    directives.widget(
        "content",
        RelatedItemsFieldWidget,
        pattern_options={
            "basePath": make_relation_root_path,
        },
    )


interface.alsoProvides(IUpdateLanguage, IFormFieldProvider)
interface.alsoProvides(IConnectTranslation, IFormFieldProvider)



from plone.app.multilingual import _
from plone.app.multilingual.browser.interfaces import IUpdateLanguage
from plone.app.multilingual.browser.utils import multilingualMoveObject
from plone.z3cform.layout import wrap_form
from z3c.form import button
from z3c.form import field
from z3c.form import form


class UpdateLanguageForm(form.Form):
    """A form to change language"""

    fields = field.Fields(IUpdateLanguage)

    ignoreContext = True

    output = None

    @button.buttonAndHandler(_("update_language", default="Update Language"))
    def handle_update(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        new_object = self.context
        language = data["language"]
        # We need to move the object to its place!!
        new_object = multilingualMoveObject(self.context, language)

        return self.request.response.redirect(
            new_object.absolute_url() + "?set_language=" + language
        )


update_language_form = wrap_form(UpdateLanguageForm)



from plone.app.multilingual.interfaces import ITG
from plone.app.multilingual.interfaces import ITranslatable
from plone.base.interfaces import ILanguage
from plone.indexer import indexer


@indexer(ITranslatable)
def itgIndexer(obj):
    return ITG(obj, None)


@indexer(ITranslatable)
def LanguageIndexer(object, **kw):
    return ILanguage(object).get_language()



from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.app.multilingual.dx.interfaces import IMultilingualAddForm
from plone.app.multilingual.events import ObjectTranslatedEvent
from plone.app.multilingual.interfaces import IMultiLanguageExtraOptionsSchema
from plone.app.multilingual.interfaces import ITG
from plone.app.uuid.utils import uuidToObject
from plone.base.interfaces import ILanguage
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.browser.add import DefaultAddView
from plone.dexterity.interfaces import IDexterityContent
from plone.registry.interfaces import IRegistry
from plone.z3cform.fieldsets.group import Group
from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import adapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.event import notify
from zope.interface import implementer
from zope.interface import Interface
from zope.traversing.interfaces import ITraversable
from zope.traversing.interfaces import TraversalError

import logging


logger = logging.getLogger(__name__)


@adapter(IFolderish, Interface)
@implementer(ITraversable)
class AddViewTraverser:
    """Add view traverser."""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.request.translation_info = dict()
        self.info = self.request.translation_info

    def traverse(self, name, ignored):
        # Populate translation info
        self.info["target_language"] = ILanguage(self.context).get_language()

        catalog = getToolByName(self.context, "portal_catalog")
        # Search source object using unrestricted API,
        # because user may be anonymous during traverse.
        brains = catalog.unrestrictedSearchResults(UID=name)
        if len(brains) != 1:
            raise TraversalError(self.context, name)
        source = brains[0]._unrestrictedGetObject()

        self.info["source_language"] = ILanguage(source).get_language()
        self.info["portal_type"] = source.portal_type
        self.info["tg"] = ITG(source)

        # If source has already been translated to this language, just redirect
        for brain in catalog.unrestrictedSearchResults(
            TranslationGroup=self.info["tg"], Language=self.info["target_language"]
        ):
            self.request.response.redirect(brain.getURL())
            return ""

        # XXX: register this adapter on dx container and a second one for AT
        if not IDexterityContent.providedBy(source):
            # we are not on DX content, assume AT
            baseUrl = self.context.absolute_url()
            url = f"{baseUrl}/@@add_at_translation?type={source.portal_type}"
            return self.request.response.redirect(url)

        # set the self.context to the place where it should be stored
        if not IFolderish.providedBy(self.context):
            self.context = self.context.__parent__

        # get the type information
        ttool = getToolByName(self.context, "portal_types")
        ti = ttool.getTypeInfo(self.info["portal_type"])

        if ti is None:
            logger.error(
                "No type information found for {}".format(self.info["portal_type"])
            )
            raise TraversalError(self.context, name)

        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            IMultiLanguageExtraOptionsSchema, prefix="plone"
        )

        if not settings.redirect_babel_view:
            add_view = None
        else:
            add_view = queryMultiAdapter(
                (self.context, self.request, ti), name="babel_view"
            )
        if add_view is None:
            add_view = queryMultiAdapter((self.context, self.request, ti))
            if add_view is None:
                raise TraversalError(self.context, name)

        add_view.__name__ = ti.factory
        return add_view


@implementer(IMultilingualAddForm)
class MultilingualAddFormGroup(Group):
    """Multilingual marked group"""


@implementer(IMultilingualAddForm)
class MultilingualAddForm(DefaultAddForm):
    babel = ViewPageTemplateFile("templates/dexterity_edit.pt")

    group_class = MultilingualAddFormGroup

    def portal_url(self):
        portal_tool = getToolByName(self.context, "portal_url", None)
        if portal_tool is not None:
            return portal_tool()
        return None

    def render(self):
        self.request["disable_border"] = True
        self.request["disable_plone.leftcolumn"] = True
        self.request["disable_plone.rightcolumn"] = True
        self.babel_content = super().render()
        return self.babel()

    def add(self, object):
        super().add(object)
        language = ILanguage(object).get_language()
        # extract UID from URL to get source object
        # probably there is a better way to do that!?
        parts = self.request.getURL().split("++")
        if not parts:
            return
        source = uuidToObject(parts[-1])
        notify(ObjectTranslatedEvent(source, object, language))

    @property
    def max_nr_of_buttons(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            IMultiLanguageExtraOptionsSchema, prefix="plone"
        )
        return settings.buttons_babel_view_up_to_nr_translations

    def _process_language_independent(self, fields, widgets):
        for field_key in fields.keys():
            if field_key in self.schema:
                schema_field = self.schema[field_key]
            else:
                # With plone.autoform, fieldnames from additional schematas
                # reference their schema by prefixing their fieldname
                # with schema.__identifier__ and then a dot as a separator
                # See autoform.txt in the autoform package
                if "." not in field_key:
                    continue
                schema_name, field_name = field_key.split(".")
                for aschema in self.additionalSchemata:
                    if schema_name == aschema.__name__ and field_name in aschema:
                        schema_field = aschema[field_name]
                        break

            if ILanguageIndependentField.providedBy(schema_field):
                widgets[field_key].addClass("languageindependent")

    def update(self):
        super().update()
        # process widgets to be shown as language independent
        self._process_language_independent(self.fields, self.widgets)
        for group in self.groups:
            self._process_language_independent(group.fields, group.widgets)


class DefaultMultilingualAddView(DefaultAddView):
    """This is the default add view as looked up by the ++add++ traversal
    namespace adapter in CMF. It is an unnamed adapter on
    (context, request, fti).

    Note that this is registered in ZCML as a simple <adapter />, but we
    also use the <class /> directive to set up security.
    """

    form = MultilingualAddForm



from Acquisition import aq_parent
from plone.app.multilingual.browser.utils import is_language_independent
from plone.app.multilingual.interfaces import ILanguageIndependentFieldsManager
from plone.app.multilingual.interfaces import ILanguageIndependentFolder
from plone.app.multilingual.interfaces import IMutableTG
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.interfaces import ITG
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.multilingual.interfaces import LANGUAGE_INDEPENDENT
from plone.app.multilingual.itg import addAttributeTG
from plone.base.interfaces import ILanguage
from plone.dexterity.interfaces import IDexterityContent
from plone.uuid.interfaces import IUUID
from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.lifecycleevent import modified
from zope.lifecycleevent.interfaces import IObjectRemovedEvent


def _reindex_site_root(obj, root, language_codes):
    for language_info in language_codes:
        lrf_to_reindex = getattr(root, language_info, None)
        to_reindex = getattr(lrf_to_reindex, obj.id, None)
        if to_reindex is not None:
            to_reindex.reindexObject()


def reindex_language_independent(ob, event):
    """Re-index language independent object for other languages

    Language independent objects are indexed once for each language with
    different, language code post-fixed, UUID for each. When ever a language
    independent object is modified in some language, it must be re-indexed
    for all the other languages as well.

    """
    if not is_language_independent(ob):
        return

    if IObjectRemovedEvent.providedBy(event):
        return

    pc = getToolByName(ob, "portal_catalog")
    parent = aq_parent(ob)

    # Re-index objects just below the language independent folder
    if ILanguageIndependentFolder.providedBy(parent):
        brains = pc.unrestrictedSearchResults(portal_type="LIF")
        for brain in brains:
            lif = brain._unrestrictedGetObject()
            if lif != parent:
                lif[ob.id].indexObject()
    # Re-index objects deeper inside language independent folder
    else:
        language_tool = getToolByName(ob, "portal_languages")
        language_codes = language_tool.supported_langs
        parent_uuid = IUUID(parent).split("-")[0] + "-"
        for code in language_codes:
            results = pc.unrestrictedSearchResults(UID=parent_uuid + code)
            # When we have results, parent has been indexed and we can reindex:
            for brain in results:
                tmp = ob.unrestrictedTraverse(brain.getPath() + "/" + ob.id)
                tmp.reindexObject()


def unindex_language_independent(ob, event):
    """Un-index language independent object for other languages

    Language independent objects are indexed once for each language with
    different, language code post-fixed, UUID for each. When ever a language
    independent object is removed in some language, we must un-indexed
    all the other languages as well.

    XXX: Removing any language independent folder will unindex contents of
    all language independent folders. When that happens, catalog clear
    and rebuild would restore contenst for the other folders.

    """
    if not is_language_independent(ob):
        return

    try:
        pc = getToolByName(ob, "portal_catalog")
    except AttributeError:
        # When we are removing the site, there is no portal_catalog:
        return

    language_tool = getToolByName(ob, "portal_languages")
    language_codes = language_tool.supported_langs
    portal = getSite()
    uuid = IUUID(ob).split("-")[0]

    for code in language_codes:
        for brain in pc.unrestrictedSearchResults(UID=uuid + "-" + code):
            portal.unrestrictedTraverse(brain.getPath()).unindexObject()
        for brain in pc.unrestrictedSearchResults(UID=uuid):
            portal.unrestrictedTraverse(brain.getPath()).unindexObject()


# Multilingual subscribers
def reindex_object(obj):
    obj.reindexObject(
        idxs=("Language", "TranslationGroup", "path", "allowedRolesAndUsers")
    )


def set_recursive_language(ob, language):
    """Set the language for this object and its children in a recursive
    manner

    """
    if is_language_independent(ob):
        ILanguage(ob).set_language(None)

    elif ILanguage(ob).get_language() != language:
        ILanguage(ob).set_language(language)
        if ITG(ob, None) is None:
            addAttributeTG(ob, None)
            ob.reindexObject(idxs=["TranslationGroup"])
        ITranslationManager(ob).update()
        reindex_object(ob)

    for child in IFolderish.providedBy(ob) and ob.items() or ():
        if ITranslatable.providedBy(child):
            set_recursive_language(child, language)


def createdEvent(obj, event):
    """Subscriber to set language on the child folder

    It can be a
    - IObjectRemovedEvent - don't do anything
    - IObjectMovedEvent
    - IObjectAddedEvent
    - IObjectCopiedEvent
    """
    if IObjectRemovedEvent.providedBy(event):
        return

    request = getattr(event.object, "REQUEST", getRequest())
    if not IPloneAppMultilingualInstalled.providedBy(request):
        return

    # On ObjectCopiedEvent and ObjectMovedEvent aq_parent(event.object) is
    # always equal to event.newParent.
    parent = aq_parent(event.object)

    # special parent handling
    if not ITranslatable.providedBy(parent):
        set_recursive_language(obj, LANGUAGE_INDEPENDENT)
        return

    # Normal use case
    # We set the tg, linking
    language = ILanguage(parent).get_language()
    set_recursive_language(obj, language)

    request = getattr(event.object, "REQUEST", getRequest())
    try:
        ti = request.translation_info
    except AttributeError:
        return

    # AT check
    portal = getSite()
    portal_factory = getToolByName(portal, "portal_factory", None)
    if (
        not IDexterityContent.providedBy(obj)
        and portal_factory is not None
        and not portal_factory.isTemporary(obj)
    ):
        return

    IMutableTG(obj).set(ti["tg"])
    modified(obj)
    tm = ITranslationManager(obj)
    old_obj = tm.get_translation(ti["source_language"])
    ILanguageIndependentFieldsManager(old_obj).copy_fields(obj)



from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.app.multilingual.dx.interfaces import MULTILINGUAL_KEY
from plone.supermodel.directives import CheckerPlugin
from plone.supermodel.directives import MetadataListDirective
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface.interfaces import IInterface


LANGUAGE_INDEPENDENT_KEY = MULTILINGUAL_KEY


class languageindependent(MetadataListDirective):
    """Directive used to mark one or more fields as 'languageindependent'"""

    key = LANGUAGE_INDEPENDENT_KEY
    value = "true"

    def factory(self, *args):
        """The languageindependent directive accepts as arguments one or more
        fieldnames (string) of fields which should be searchable.
        """
        if not args:
            raise TypeError(
                "The languageindependent directive expects at " "least one argument."
            )

        form_interface = Interface
        if IInterface.providedBy(args[0]):
            form_interface = args[0]
            args = args[1:]
        return [(form_interface, field, self.value) for field in args]


class LanguageIndependentFieldsPlugin(CheckerPlugin):
    key = LANGUAGE_INDEPENDENT_KEY

    def __call__(self):
        schema = self.schema
        for fieldName in self.check():
            alsoProvides(schema[fieldName], ILanguageIndependentField)

    def fieldNames(self):
        if self.value is None:
            return
        for taggedValue in self.value:
            yield taggedValue[1]


__all__ = ("languageindependent", "LanguageIndependentFieldsPlugin")



from plone.app.multilingual import api
from plone.app.multilingual.browser.vocabularies import untranslated_languages
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.testing import PAM_FUNCTIONAL_TESTING
from plone.dexterity.utils import createContentInContainer
from zope.interface import alsoProvides

import unittest


class TestVocabularies(unittest.TestCase):
    layer = PAM_FUNCTIONAL_TESTING

    def setUp(self):
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)
        self.portal = self.layer["portal"]

    def test_content_is_translated_into_all_languages(self):
        a_ac = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        api.translate(a_ac, "en")
        api.translate(a_ac, "es")

        self.assertEqual(len(untranslated_languages(a_ac)), 0)

    def test_content_is_not_translated_to_any_language(self):
        a_ac = createContentInContainer(self.portal, "Document", title="Test document")

        languages = untranslated_languages(a_ac).by_token.keys()

        self.assertEqual(len(languages), 3)
        self.assertIn("ca", languages)
        self.assertIn("es", languages)
        self.assertIn("en", languages)



from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.multilingual.testing import PAM_FUNCTIONAL_TESTING
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.dexterity.utils import createContentInContainer
from plone.testing._z2_testbrowser import Browser
from zope.interface import alsoProvides

import transaction
import unittest


class TestForm(unittest.TestCase):
    layer = PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)
        # Setup test browser
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization", f"Basic {SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}"
        )

    def test_all_translation_links_are_shown(self):
        a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        transaction.commit()

        self.browser.open(a_ca.absolute_url())
        self.assertIn("plone-contentmenu-multilingual", self.browser.contents)
        self.assertIn("translate_into_es", self.browser.contents)
        self.assertIn("translate_into_en", self.browser.contents)

    def test_translation_form_creates_translation(self):
        a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        transaction.commit()

        # Translate content
        self.browser.open(a_ca.absolute_url() + "/@@create_translation?language=en")

        # Fill in translation details
        self.browser.getControl(
            name="form.widgets.IDublinCore.title"
        ).value = "Test document"
        self.browser.getControl(name="form.buttons.save").click()

        self.portal._p_jar.sync()

        self.assertIn("test-document", self.portal["en"].objectIds())

        self.browser.open(a_ca.absolute_url())
        self.assertIn("plone-contentmenu-multilingual", self.browser.contents)
        self.assertIn("translate_into_es", self.browser.contents)
        self.assertNotIn("translate_into_en", self.browser.contents)

    def test_translation_form_prevents_translating_twice(self):
        a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        transaction.commit()

        # Translate content
        self.browser.open(a_ca.absolute_url() + "/@@create_translation?language=en")

        # Save ++add++translation... URL
        add_translation_url = self.browser.url

        # Fill in translation details
        self.browser.getControl(
            name="form.widgets.IDublinCore.title"
        ).value = "Test document"
        self.browser.getControl(name="form.buttons.save").click()

        self.portal._p_jar.sync()

        # Revisit the saved ++add++translation... URL
        self.browser.open(add_translation_url)

        # Which should now redirect to the created translation
        self.assertEqual(
            self.portal["en"]["test-document"].absolute_url(), self.browser.url
        )

    def test_translation_can_be_unregistered(self):
        a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        transaction.commit()

        # Create translation
        self.browser.open(a_ca.absolute_url() + "/@@create_translation?language=en")

        # Fill in translation details
        self.browser.getControl(
            name="form.widgets.IDublinCore.title"
        ).value = "Test document"
        self.browser.getControl(name="form.buttons.save").click()

        # Unregister translation
        self.browser.open(
            a_ca.absolute_url()
            + f"/disconnect_translation?came_from={a_ca.UID()}&language=en"
        )
        self.browser.getForm(index=1).submit()

        self.portal._p_jar.sync()

        self.assertNotIn("en", ITranslationManager(a_ca).get_translations())

        # self.assertEqual(
        #     self.browser.getControl(name="form.widgets.languages:list").value,
        #     ['en'])
        # self.browser.getControl(name='form.buttons.unlink').click()
        # self.assertEqual(
        #     self.browser.getControl(name="form.widgets.languages:list").value,
        #     [])

        # Translation is unregistered
        self.browser.open(a_ca.absolute_url())
        self.assertIn("plone-contentmenu-multilingual", self.browser.contents)
        self.assertIn("translate_into_es", self.browser.contents)
        self.assertIn("translate_into_en", self.browser.contents)

        transaction.commit()

        # Content is still available
        self.assertIn("test-document", self.portal["en"].contentIds())

    # Removed because its testes on robot framework with the relation pattern
    # def test_registering_translation(self):
    #     a_ca = createContentInContainer(
    #         self.portal['ca'], 'Document', title=u"Test document")

    #     b_ca = createContentInContainer(
    #         self.portal['ca'], 'Document', title=u"Test document")

    #     transaction.commit()

    #     # Register translation
    #     self.browser.open(
    #         a_ca.absolute_url() + '/add_translations')
    #     self.assertEqual(self.browser.getControl(
    #         name="form.widgets.language:list").options, ['en', 'es'])

    #     # Fill in form
    #     form = self.browser.getForm(index=1)
    #     form.mech_form.new_control(
    #         type='radio',
    #         name='form.widgets.content:list',
    #         attrs=dict(checked='checked',
    #                    value='%s' % '/'.join(b_ca.getPhysicalPath()),
    #                    id='form-widgets-content-0'))
    #     self.browser.getControl(
    #         name="form.widgets.language:list").value = ['en']
    #     self.browser.getControl(
    #         name='form.buttons.add_translations').click()

    #     # Language is removed from nontranslated languages
    #     self.assertEqual(self.browser.getControl(
    #         name="form.widgets.language:list").options, ['es'])

    #     # And translation can be unregistered
    #     self.browser.open(a_ca.absolute_url() + '/remove_translations')
    #     self.assertEqual(self.browser.getControl(
    #         name="form.widgets.languages:list").value, ['en'])

    def test_translation_can_be_removed_by_deleting(self):
        a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        transaction.commit()

        # Translate content
        self.browser.open(a_ca.absolute_url() + "/@@create_translation?language=en")

        # Fill in translation details
        self.browser.getControl(
            name="form.widgets.IDublinCore.title"
        ).value = "Test document"
        self.browser.getControl(name="form.buttons.save").click()

        # Remove translation
        self.browser.open(
            ITranslationManager(a_ca).get_translation("en").absolute_url()
            + "/delete_confirmation"
        )
        self.browser.getControl(name="form.buttons.Delete").click()

        self.portal._p_jar.sync()
        self.assertNotIn("en", ITranslationManager(a_ca).get_translations())

    def test_folderish_content_can_be_translated(self):
        createContentInContainer(self.portal["ca"], "Folder", title="Test folder")

        transaction.commit()

        self.browser.open(
            self.portal.absolute_url()
            + "/ca/test-folder/@@create_translation?language=en"
        )

        self.browser.getControl(
            name="form.widgets.IDublinCore.title"
        ).value = "Test folder"
        self.browser.getControl(name="form.buttons.save").click()

        self.portal._p_jar.sync()

        self.assertIn("test-folder", self.portal["en"].objectIds())

    def test_content_in_folders_can_be_translated(self):
        af_ca = createContentInContainer(
            self.portal["ca"], "Folder", title="Test folder"
        )

        b_ca = createContentInContainer(
            self.portal["ca"]["test-folder"], "Document", title="Test document"
        )

        transaction.commit()

        self.browser.open(
            af_ca.absolute_url() + "/" + b_ca.id + "/@@create_translation?language=en"
        )

        self.browser.getControl(
            name="form.widgets.IDublinCore.title"
        ).value = "Test folder"
        self.browser.getControl(name="form.buttons.save").click()

        self.portal._p_jar.sync()

        self.assertIn("test-folder", self.portal["en"].objectIds())



from plone.app.multilingual import isLPinstalled
from plone.app.multilingual.browser.migrator import portal_types_blacklist
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.multilingual.interfaces import IMultiLanguageExtraOptionsSchema
from plone.app.registry.browser import controlpanel
from plone.app.uuid.utils import uuidToObject
from plone.base.interfaces import ILanguage
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.controlpanel.browser.language import LanguageControlPanelForm
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from zc.relation.interfaces import ICatalog as IRelationCatalog
from zope.component import getUtility
from zope.i18nmessageid import MessageFactory
from zope.interface.interfaces import ComponentLookupError
from zope.schema.interfaces import IVocabularyFactory

import json


_ = MessageFactory("plone.app.multilingual")


class LanguageControlPanelFormPAM(LanguageControlPanelForm):
    """A modified language control panel, allows selecting multiple languages."""

    label = _("Multilingual Settings")
    description = _(
        "pam_controlpanel_description",
        default="All the configuration of " "a multilingual Plone site",
    )
    schema = IMultiLanguageExtraOptionsSchema

    @button.buttonAndHandler(_("Save"), name="save")
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        # We need to check if the default language is in available languages
        if (
            "default_language" in data
            and "available_languages" in data
            and data["default_language"] not in data["available_languages"]
        ):
            IStatusMessage(self.request).addStatusMessage(
                _("Default language not in available languages"), "error"
            )

            # e = Invalid(_(u"Default language not in available languages"))
            # raise WidgetActionExecutionError('default_language', e)
            return

        self.applyChanges(data)

        setupTool = SetupMultilingualSite()
        output = setupTool.setupSite(self.context)
        self.status += output

    @button.buttonAndHandler(_("Cancel"), name="cancel")
    def handleCancel(self, action):
        IStatusMessage(self.request).addStatusMessage(_("Changes canceled."), "info")
        self.request.response.redirect(
            f"{self.context.absolute_url()}/{self.control_panel_view}"
        )


class LanguageControlPanel(controlpanel.ControlPanelFormWrapper):
    form = LanguageControlPanelFormPAM
    index = ViewPageTemplateFile("templates/controlpanel.pt")
    isLPinstalled = isLPinstalled


class MigrationView(BrowserView):
    """The view for display the migration information, actions and results"""

    __call__ = ViewPageTemplateFile("templates/migration.pt")

    isLPinstalled = isLPinstalled
    portal_types_blacklist = portal_types_blacklist
    try:
        catalog = getUtility(IRelationCatalog)
        hasRelationCatalog = True
    except ComponentLookupError:
        hasRelationCatalog = False


class MigrationViewAfter(BrowserView):
    """The view for display the migration information, actions and results"""

    __call__ = ViewPageTemplateFile("templates/cleanup.pt")

    isLPinstalled = isLPinstalled


class multilingualMapViewJSON(BrowserView):
    """Helper view to get json translations"""

    def __call__(self):
        """Get the JSON information about based on a nodeId"""

        # We get the language we are looking for
        lang = ""
        tool = getToolByName(self.context, "portal_languages", None)
        if "lang" in self.request:
            lang = self.request["lang"]

        if lang == "":
            lang = tool.getDefaultLanguage()

        # We want all or just the missing translations elements
        if "all" in self.request:
            get_all = self.request["all"] == "true"
        else:
            get_all = True

        # Which is the root nodeId
        folder_path = ""
        if "nodeId" in self.request:
            # We get the used UUID
            nodeId = self.request["nodeId"]
            if nodeId != "root":
                new_root = uuidToObject(nodeId)
                if ILanguage(new_root).get_language() == lang:
                    folder_path = "/".join(new_root.getPhysicalPath())
        if folder_path == "":
            # We get the root folder
            root = getToolByName(self.context, "portal_url")
            root = root.getPortalObject()
            folder_path = "/".join(root.getPhysicalPath())

        self.request.response.setHeader(
            "Content-type", "application/json; charset=utf-8"
        )

        pcatalog = getToolByName(self.context, "portal_catalog")
        query = {}
        query["path"] = {"query": folder_path, "depth": 1}
        query["sort_on"] = "sortable_title"
        query["sort_order"] = "ascending"
        query["Language"] = lang
        search_results = pcatalog.searchResults(query)
        resultat = {"id": "root", "name": folder_path, "data": {}, "children": []}
        supported_languages = tool.getSupportedLanguages()
        for sr in search_results:
            # We want to know the translated and missing elements
            translations = {}
            if "TranslationGroup" in sr:
                # We look for the brain for each translation
                brains = pcatalog.unrestrictedSearchResults(
                    TranslationGroup=sr["TranslationGroup"]
                )
                languages = {}
                for brain in brains:
                    languages[brain.Language] = brain.UID
                for lang in supported_languages:
                    if lang in languages.keys():
                        translated_obj = uuidToObject(languages[lang])
                        translations[lang] = {
                            "url": translated_obj.absolute_url(),
                            "title": translated_obj.getId(),
                        }
                    else:
                        url_to_create = (
                            f"{sr.getURL()}/@@create_translation?language={lang}"
                        )
                        translations[lang] = {
                            "url": url_to_create,
                            "title": _("Not translated"),
                        }
            if get_all:
                resultat["children"].append(
                    {
                        "id": sr["UID"],
                        "name": sr["Title"],
                        "data": translations,
                        "children": [],
                    }
                )
            else:
                pass
        return json.dumps(resultat)


class multilingualMapView(BrowserView):
    """The view for display the current multilingual map for the site"""

    __call__ = ViewPageTemplateFile("templates/mmap.pt")

    def languages(self):
        langs = getUtility(
            IVocabularyFactory,
            name="plone.app.multilingual.vocabularies"
            ".AllAvailableLanguageVocabulary",
        )
        tool = getToolByName(self.context, "portal_languages", None)
        lang = tool.getDefaultLanguage()
        return {"default": lang, "languages": langs(self.context)}

    def canonicals(self):
        """We get all the canonicals and see which translations are
        missing"""
        # Get the language
        tool = getToolByName(self.context, "portal_languages", None)
        pcatalog = getToolByName(self.context, "portal_catalog", None)
        languages = tool.getSupportedLanguages()
        num_lang = len(languages)
        # Get the canonicals
        # Needs to be optimized
        not_full_translations = []
        already_added_canonicals = []
        brains = pcatalog.searchResults()
        for brain in brains:
            if not isinstance(brain.TranslationGroup, str):
                # is alone, with a Missing.Value
                missing_languages = [
                    lang for lang in languages if lang != brain.Language
                ]
                translations = [
                    {
                        "url": brain.getURL(),
                        "path": brain.getPath(),
                        "lang": brain.Language,
                    }
                ]
                not_full_translations.append(
                    {
                        "id": "None",
                        "last_url": brain.getURL(),
                        "missing": missing_languages,
                        "translated": translations,
                    }
                )
            elif isinstance(brain.TranslationGroup, str):
                tg = brain.TranslationGroup
                brains_tg = pcatalog.searchResults(TranslationGroup=tg)
                if len(brains_tg) < num_lang and tg not in already_added_canonicals:
                    translated_languages = [a.Language for a in brains_tg]
                    missing_languages = [
                        lang for lang in languages if lang not in translated_languages
                    ]
                    translations = []
                    last_url = ""
                    for brain_tg in brains_tg:
                        last_url = brain_tg.getURL()
                        translations.append(
                            {
                                "url": brain_tg.getURL(),
                                "path": brain_tg.getPath(),
                                "lang": brain_tg.Language,
                            }
                        )

                    not_full_translations.append(
                        {
                            "id": tg,
                            "last_url": last_url,
                            "missing": missing_languages,
                            "translated": translations,
                        }
                    )
                already_added_canonicals.append(tg)
        return not_full_translations

    isLPinstalled = isLPinstalled



from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.supermodel.interfaces import IFieldMetadataHandler
from plone.supermodel.utils import ns
from zope.interface import alsoProvides
from zope.interface import implementer


@implementer(IFieldMetadataHandler)
class LanguageIndependentFieldMetadataHandler:
    """Define the ``lingua`` namespace.

    This lets you write lingua:independent="true" on a field to mark it as
    a language independent field.
    """

    namespace = "http://namespaces.plone.org/supermodel/lingua"
    prefix = "lingua"

    def read(self, fieldNode, schema, field):
        independent = fieldNode.get(ns("independent", self.namespace))
        if independent is not None and independent.lower() in (
            "true",
            "on",
            "yes",
            "y",
            "1",
        ):
            alsoProvides(field, ILanguageIndependentField)

    def write(self, fieldNode, schema, field):
        if ILanguageIndependentField.providedBy(field):
            fieldNode.set(ns("independent", self.namespace), "true")



from Acquisition import aq_base
from plone.app.multilingual.interfaces import ATTRIBUTE_NAME
from plone.app.multilingual.interfaces import IMutableTG
from plone.app.multilingual.interfaces import ITG
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import NOTG
from plone.uuid.interfaces import IUUIDGenerator
from zope.component import adapter
from zope.component import queryUtility
from zope.interface import implementer
from zope.lifecycleevent.interfaces import IObjectCopiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent


@implementer(ITG)
@adapter(ITranslatable)
def attributeTG(context):
    return getattr(context, ATTRIBUTE_NAME, None)


@implementer(IMutableTG)
@adapter(ITranslatable)
class MutableAttributeTG:
    def __init__(self, context):
        self.context = context

    def get(self):
        return getattr(self.context, ATTRIBUTE_NAME, None)

    def set(self, tg):
        if tg == NOTG:
            generator = queryUtility(IUUIDGenerator)
            tg = generator()
        tg = str(tg)
        setattr(self.context, ATTRIBUTE_NAME, tg)


@adapter(ITranslatable, IObjectCreatedEvent)
def addAttributeTG(obj, event):
    if not IObjectCopiedEvent.providedBy(event) and getattr(
        aq_base(obj), ATTRIBUTE_NAME, None
    ):
        return  # defensive: keep existing TG on non-copy create

    generator = queryUtility(IUUIDGenerator)
    if generator is None:
        return

    tg = generator()
    if not tg:
        return

    setattr(obj, ATTRIBUTE_NAME, tg)



from plone.app.multilingual.interfaces import ITranslatable
from zope.interface import Interface


MULTILINGUAL_KEY = "plone.app.multilingual.languageindependent"


class IDexterityTranslatable(ITranslatable):
    """special marker for dexterity"""


class ILanguageIndependentField(Interface):
    """Marker interface for language independent fields"""


class IMultilingualAddForm(Interface):
    """Marker Interface for multilingual add forms

    This marker is intended to be provided by the main multilingual add form
    ans to be provided by the groups (aka fieldset) forms
    (plone.z3cform.fieldsets.group.Group). This is neccessary in order to
    make the special template renderer for ILanguageIndependentField fields
    work in fieldset tabs too.
    """



from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView


class LanguageSwitcher(BrowserView):
    def __call__(self):
        """Redirect to the preferred language top-level folder.

        If no folder for preferred language exists, redirect to default
        language.

        Copy from LinguaPlone
        """
        context = aq_inner(self.context)
        plt = getToolByName(context, "portal_languages")
        pref = plt.getPreferredLanguage(self.request)
        default = plt.getDefaultLanguage()
        ids = self.context.keys()
        target = (pref in ids) and pref or default
        url = f"{context.absolute_url()}/{target}"

        # We need to set the language cookie on the first response or it will
        # be set on the frontpage itself, making it uncachable
        langCookie = self.request.cookies.get("I18N_LANGUAGE")
        if not langCookie or langCookie != target:
            self.request.response.setCookie("I18N_LANGUAGE", target, path="/")

        self.request.response.redirect(url, status=302)



from plone.app.content.namechooser import NormalizingNameChooser
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.multilingual.interfaces import ILanguageRootFolder
from plone.app.multilingual.interfaces import ITranslationIdChooser
from plone.dexterity.content import Container
from zope.component import adapter
from zope.container.interfaces import INameChooser
from zope.deprecation import deprecated
from zope.interface import implementer


_marker = object()


@implementer(INameChooser)
@adapter(ILanguageRootFolder)
class LRFNameChooser(NormalizingNameChooser):
    """Special name chooser to fix issue where createContentInContainer is
    used to create a new content into LRF with an id, which exists already
    in the parent folder.

    """

    def chooseName(self, name, object):
        chosen = super().chooseName(name, object)
        if chosen in self.context.objectIds():
            old_id = getattr(object, "id", None)
            object.id = chosen
            chooser = ITranslationIdChooser(object)
            chosen = chooser(self.context, self.context.getId())
            object.id = old_id
        return chosen


@implementer(ILanguageRootFolder, INavigationRoot)
class LanguageRootFolder(Container):
    """Deprecated LanguageRootFolder custom base class"""


deprecated(
    "LanguageRootFolder", "LanguageRootFolders should be migrate to DexterityContainers"
)



from AccessControl.SecurityManagement import getSecurityManager
from Acquisition import aq_chain
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.i18n.locales.browser.selector import LanguageSelector
from plone.app.multilingual.browser.selector import LanguageSelectorViewlet
from plone.app.multilingual.interfaces import ILanguageIndependentFolder
from plone.app.multilingual.interfaces import IMultiLanguageExtraOptionsSchema
from plone.app.multilingual.interfaces import ITranslationLocator
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.multilingual.manager import TranslationManager
from plone.base.interfaces import ILanguage
from plone.i18n.locales.interfaces import IContentLanguageAvailability
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.hooks import getSite


def is_language_independent(ob):
    if not ILanguageIndependentFolder.providedBy(ob):
        for element in aq_chain(ob):
            if ILanguageIndependentFolder.providedBy(element):
                return True
    return False


class BabelUtils(BrowserView):
    def __init__(self, context, request):
        super().__init__(context, request)
        portal_state = getMultiAdapter((context, request), name="plone_portal_state")
        self.portal_url = portal_state.portal_url()
        # If there is any translation_info lets use it
        try:
            self.group = TranslationManager(request.translation_info["tg"])
        except AttributeError:
            self.group = ITranslationManager(self.context)

    def getGroup(self):
        return self.group

    def getTranslatedLanguages(self):
        return self.group.get_translated_languages()

    def getPortal(self):
        portal_url = getToolByName(self.context, "portal_url")
        return portal_url

    def objToTranslate(self):
        return self.context

    def gtenabled(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            IMultiLanguageExtraOptionsSchema, prefix="plone"
        )
        key = settings.google_translation_key
        return key is not None and len(key.strip()) > 0

    def languages(self):
        """Deprecated"""
        context = aq_inner(self.context)

        ls = LanguageSelector(context, self.request, None, None)
        ls.update()
        results = ls.languages()

        supported_langs = [v["code"] for v in results]
        missing = {str(c) for c in supported_langs}

        lsv = LanguageSelectorViewlet(context, self.request, None, None)
        translations = lsv._translations(missing)

        # We want to see the babel_view
        append_path = (
            "",
            "babel_view",
        )
        non_viewable = set()
        for data in results:
            code = str(data["code"])
            data["translated"] = code in translations.keys()

            appendtourl = "/".join(append_path)

            if data["translated"]:
                trans, direct, has_view_permission = translations[code]
                if not has_view_permission:
                    # shortcut if the user cannot see the item
                    non_viewable.add(data["code"])
                    continue
                data["url"] = trans.absolute_url() + appendtourl
            else:
                non_viewable.add(data["code"])

        # filter out non-viewable items
        results = [r for r in results if r["code"] not in non_viewable]

        return results

    def translated_languages(self):
        context = aq_inner(self.context)
        tool = getToolByName(context, "portal_languages", None)
        checkPermission = getSecurityManager().checkPermission
        translations = self.group.get_translations()
        translated_info = [
            dict(
                code=key,
                info=tool.getAvailableLanguageInformation()[key],
                obj=translations[key],
            )
            for key in translations
        ]

        default_language = tool.getDefaultLanguage()

        translated_shown = []

        for lang_info in translated_info:
            # Mark the default language as the first translation shown
            if lang_info["code"] == default_language:
                lang_info["isDefault"] = True
            else:
                lang_info["isDefault"] = False

            # Remove the translation of the content currently being
            # translated In case it's temporal we show as language is not
            # already set on AT
            portal_factory = getToolByName(self.context, "portal_factory", None)
            context_language = ILanguage(context).get_language()
            if (
                portal_factory is None or not portal_factory.isTemporary(self.context)
            ) and lang_info["code"] == context_language:
                continue

            # Remove the translation in case the translator user does not
            # have permissions over it
            has_view_permission = bool(checkPermission("View", lang_info["obj"]))
            if not has_view_permission:
                continue

            translated_shown.append(lang_info)
        return translated_shown

    def current_language_name(self):
        """Get the current language native name"""
        adapted = ILanguage(self.context)
        lang_code = adapted.get_language()
        util = getUtility(IContentLanguageAvailability)
        data = util.getLanguages(True)
        lang_info = data.get(lang_code)
        return lang_info.get("native", None) or lang_info.get("name")

    def max_nr_of_buttons(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            IMultiLanguageExtraOptionsSchema, prefix="plone"
        )
        return settings.buttons_babel_view_up_to_nr_translations


def multilingualMoveObject(content, language):
    """Move content object and its contained objects to a new language folder
    Also set the language on all the content moved

    """
    if is_language_independent(content):
        # Language independent content will be created into language roo
        target_folder = getattr(getSite(), language)
    else:
        target_folder = ITranslationLocator(content)(language)

    parent = aq_parent(content)

    copy_data = parent.manage_cutObjects(content.getId())
    list_ids = target_folder.manage_pasteObjects(copy_data)
    new_id = list_ids[0]["new_id"]
    new_object = target_folder[new_id]

    new_object.reindexObject()

    return new_object



from logging import getLogger
from plone.app.dexterity.behaviors.exclfromnav import IExcludeFromNavigation
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.multilingual import _
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.multilingual.interfaces import LANGUAGE_INDEPENDENT
from plone.app.multilingual.subscriber import set_recursive_language
from plone.base.interfaces import ILanguage
from plone.dexterity.interfaces import IDexterityFTI
from plone.i18n.locales.languages import _combinedlanguagelist
from plone.i18n.locales.languages import _languagelist
from Products.CMFCore.utils import getToolByName
from plone.base.utils import unrestricted_construct_instance
from Products.Five import BrowserView
from zope.component.hooks import getSite
from zope.event import notify
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.lifecycleevent import modified


logger = getLogger("plone.app.multilingual")


# Setup view imported from LinguaPlone
class SetupView(BrowserView):
    def __init__(self, context, request):
        super().__init__(context, request)

    def __call__(self, forceOneLanguage=False, forceMovingAndSetting=True):
        setupTool = SetupMultilingualSite()
        return setupTool.setupSite(self.context, forceOneLanguage)


class SetupMultilingualSite:
    # portal_type that is added as root language folder
    folder_type = "LRF"

    # portal_type that is added as language independent asset folder
    folder_type_language_independent = "LIF"

    def __init__(self, context=None):
        self.context = context

        self.folders = {}
        self.languages = []
        self.defaultLanguage = None
        self.previousDefaultPageId = None

    def setupSite(self, context, forceOneLanguage=False):
        """This method is called from the control panel to setup a site in
        order to have root language folders and a redirect view. It needs to be
        executed every time a new language is added to the Available Languages.

        """
        self.context = context
        self.folders = {}

        self.ensure_translatable(self.folder_type)
        self.ensure_translatable(self.folder_type_language_independent)

        language_tool = getToolByName(self.context, "portal_languages")
        self.languages = languages = language_tool.getSupportedLanguages()
        self.defaultLanguage = language_tool.getDefaultLanguage()

        if len(languages) == 1 and not forceOneLanguage:
            return "Only one supported language configured."

        doneSomething = False
        available = language_tool.getAvailableLanguages()
        for language in languages:
            info = available[language]
            name = info.get("native", info.get("name"))
            doneSomething += self.setUpLanguage(language, name)

        doneSomething += self.linkTranslations()
        doneSomething += self.removePortalDefaultPage()
        doneSomething += self.setupLanguageSwitcher()
        self.set_default_language_content()

        if not doneSomething:
            return "Nothing done."
        else:
            return "Setup of language root folders on Plone site '%s'" % (
                self.context.getId()
            )

    def linkTranslations(self):
        """Links the translations of the default language Folders"""
        doneSomething = False

        try:
            canonical = ITranslationManager(self.folders[self.defaultLanguage])
        except TypeError as e:
            raise TypeError(str(e) + " Are your folders ITranslatable?")

        for language in self.languages:
            if language == self.defaultLanguage:
                continue
            if not canonical.has_translation(language):
                language_folder = self.folders[language]
                canonical.register_translation(language, language_folder)
                doneSomething = True

        if doneSomething:
            logger.info("Translations linked.")

        return doneSomething

    def set_default_language_content(self):
        """Set default language on root to language independent"""
        portal = getSite()
        defaultLanguage = LANGUAGE_INDEPENDENT

        for id_ in portal.objectIds():
            if all(
                [
                    id_ not in _languagelist,
                    id_ not in _combinedlanguagelist,
                    ITranslatable.providedBy(portal[id_]),
                ]
            ):
                set_recursive_language(portal[id_], defaultLanguage)

    def setUpLanguage(self, code, name):
        """Create the language folders on top of the site"""
        doneSomething = False

        if code == "id":
            folderId = "id-id"
        else:
            folderId = str(code)

        folder = getattr(self.context, folderId, None)
        wftool = getToolByName(self.context, "portal_workflow")

        assets_folder_id = translate(
            _("assets_folder_id", default="assets"),
            domain="plone",
            target_language=folderId,
        )
        assets_folder_title = translate(
            _("assets_folder_title", default="Assets"),
            domain="plone",
            target_language=folderId,
        )

        if folder is None:
            unrestricted_construct_instance(self.folder_type, self.context, folderId)
            unrestricted_construct_instance(
                self.folder_type_language_independent,
                self.context[folderId],
                assets_folder_id,
            )

            folder = self.context[folderId]

            ILanguage(folder).set_language(code)
            folder.setTitle(name)

            ILanguage(folder[assets_folder_id]).set_language(code)
            folder[assets_folder_id].setTitle(assets_folder_title)

            # This assumes a direct 'publish' transition from the initial state
            # We are going to check if its private and has publish action for
            # the out of the box case otherwise don't do anything
            state = wftool.getInfoFor(folder, "review_state", None)
            available_transitions = [t["id"] for t in wftool.getTransitionsFor(folder)]
            if state != "published" and "publish" in available_transitions:
                wftool.doActionFor(folder, "publish")

            state = wftool.getInfoFor(folder[assets_folder_id], "review_state", None)
            available_transitions = [
                t["id"] for t in wftool.getTransitionsFor(folder[assets_folder_id])
            ]  # noqa
            if state != "published" and "publish" in available_transitions:
                wftool.doActionFor(folder[assets_folder_id], "publish")

            # Exclude folder from navigation (if applicable)
            adapter = IExcludeFromNavigation(folder, None)
            if adapter is not None:
                adapter.exclude_from_nav = True

            adapter = IExcludeFromNavigation(folder[assets_folder_id], None)
            if adapter is not None:
                adapter.exclude_from_nav = True

            # We've modified the object; reindex.
            notify(modified(folder))
            notify(modified(folder[assets_folder_id]))

            doneSomething = True
            logger.info(f"Added '{code}' folder: {folderId}")

        self.folders[code] = folder
        if not INavigationRoot.providedBy(folder):
            alsoProvides(folder, INavigationRoot)

            doneSomething = True
            logger.info("INavigationRoot setup on folder '%s'" % code)

        return doneSomething

    def removePortalDefaultPage(self):
        """Remove the default page of the site"""

        defaultPageId = self.context.getDefaultPage()
        if not defaultPageId:
            return False

        self.previousDefaultPageId = defaultPageId
        self.context.setDefaultPage(None)
        self.context.reindexObject()

        logger.info("Portal default page removed.")
        return True

    def resetDefaultPage(self):
        """Maintain the default page of the site on the language it was defined"""
        previousDefaultPage = getattr(self.context, self.previousDefaultPageId)
        languageWrapped = ILanguage(previousDefaultPage, None)

        # If the previous default page cannot be adapted, do nothing.
        # This might be the case if it is a Python Script or other non-portal
        # content
        if languageWrapped is None:
            return False
        language = languageWrapped.get_language()
        pageId = self.previousDefaultPageId

        # test language neutral
        if language == "":
            language = self.defaultLanguage
        target = self.folders[language]
        objects = self.context.manage_cutObjects(pageId)

        try:
            target.manage_pasteObjects(objects)
        except ValueError as exc:
            # This portal_type may not be allowed.  This should not be
            # fatal, so we only log a warning.
            logger.warn(
                (
                    "Could not move default page '{:s}' j" "to folder '{:s}': {:s}"
                ).format(pageId, target.getId(), exc)
            )
            return False

        target.setDefaultPage(pageId)
        target.reindexObject()
        defaultPage = getattr(target, pageId)
        defaultPage.reindexObject()

        logger.info(f"Moved default page '{pageId}' to folder '{target.getId()}'.")

        return True

    def setupLanguageSwitcher(self):
        """
        Add the new default page
        """
        doneSomething = False

        tt = getToolByName(self.context, "portal_types")
        site = tt["Plone Site"]

        methods = tuple(site.view_methods)
        if "language-switcher" not in methods:
            site.view_methods = methods + ("language-switcher",)
            site.default_view = "language-switcher"
            self.context.reindexObject()

            doneSomething = True
            logger.info("Root language switcher set up.")

        return doneSomething

    def ensure_translatable(self, type_):
        types_tool = getToolByName(self.context, "portal_types")
        fti = getattr(types_tool, type_)

        if IDexterityFTI.providedBy(fti):
            behaviors = list(fti.behaviors)
            if "plone.translatable" not in behaviors:
                behaviors.append("plone.translatable")
            fti._updateProperty("behaviors", tuple(behaviors))



from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.testing import PAM_FUNCTIONAL_TESTING
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.dexterity.utils import createContentInContainer
from plone.testing._z2_testbrowser import Browser
from zope.interface import alsoProvides

import transaction
import unittest


class TestMenu(unittest.TestCase):
    layer = PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)
        # Setup test browser
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            f"Basic {SITE_OWNER_NAME:s}:{SITE_OWNER_PASSWORD:s}",
        )

        # Create sample document in 'en' and index it into catalog
        self.a_ca = createContentInContainer(
            self.portal["ca"], "Document", title="Test document"
        )

        transaction.commit()

    def test_menu_is_visible(self):
        self.browser.open(self.a_ca.absolute_url())
        # 'Translate' in Catalan:
        self.assertIn("Tradueix", self.browser.contents)

    def test_menu_contains_translatable_entries(self):
        self.browser.open(self.a_ca.absolute_url())
        self.assertIn("translate_into_es", self.browser.contents)
        self.assertIn("translate_into_en", self.browser.contents)

    def test_menu_does_not_contain_translated_entries(self):
        self.browser.open(self.a_ca.absolute_url())
        self.assertNotIn("translate_into_ca", self.browser.contents)

    def test_menu_does_not_appear_without_ITranslatable(self):
        self.assertFalse(ITranslatable.providedBy(self.portal))
        self.browser.open(self.portal.absolute_url() + "/folder_listing")
        self.assertNotIn("Translate", self.browser.contents)

    def test_menu_is_visible_on_folder_default_page(self):
        createContentInContainer(self.portal["ca"], "Folder", title="Test folder")
        createContentInContainer(
            self.portal["ca"]["test-folder"], "Document", title="Test document"
        )
        self.portal["ca"]["test-folder"].setDefaultPage("test-document")

        transaction.commit()

        self.browser.open(self.portal["ca"]["test-folder"].absolute_url())
        self.assertIn("test-folder/@@create_translation", self.browser.contents)
        self.assertIn("test-document/@@create_translation", self.browser.contents)

    def test_menu_is_not_visible_without_permission(self):
        # Add a new user without Owner, Editor or Manager role on
        # self.a_ca document
        self.portal.portal_registration.addMember(
            "a_user",
            "a_password",
            [
                "Reader",
                "Contributor",
                "Reviewer",
            ],
        )

        transaction.commit()

        browser = Browser(self.layer["app"])
        browser.addHeader("Authorization", "Basic a_user:a_password")

        browser.open(self.a_ca.absolute_url())
        self.assertNotIn("plone-contentmenu-multilingual", browser.contents)



from plone.app.multilingual.testing import PAM_ROBOT_TESTING
from plone.testing import layered

import robotsuite
import unittest


UNIT_TEST_LEVEL = 1
INTEGRATION_TEST_LEVEL = 2
FUNCTIONAL_TEST_LEVEL = 3
ROBOT_TEST_LEVEL = 5


def test_suite():
    suite = unittest.TestSuite()
    suite.level = ROBOT_TEST_LEVEL
    suite.addTests(
        [
            layered(
                robotsuite.RobotTestSuite("robot", noncritical=["unstable"]),
                layer=PAM_ROBOT_TESTING,
            ),
        ]
    )
    return suite



from plone.app.multilingual.browser.utils import is_language_independent
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.vocabularies.catalog import CatalogVocabularyFactory
from plone.base.interfaces import ILanguage
from plone.i18n.locales.interfaces import ILanguageAvailability
from Products.CMFCore.utils import getToolByName
from zope.component import getGlobalSiteManager
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@provider(IContextSourceBinder)
def untranslated_languages(context):
    language_tool = getToolByName(context, "portal_languages")
    language_infos = language_tool.getAvailableLanguages()
    available_portal_languages = language_tool.supported_langs
    manager = ITranslationManager(context)
    translated_languages = manager.get_translated_languages()
    if is_language_independent(context):
        translated_languages = []
    languages = []
    for lang in available_portal_languages:
        if lang not in translated_languages:
            native = language_infos[lang].get("native", None)
            name = language_infos[lang].get("name", lang)
            languages.append(SimpleVocabulary.createTerm(lang, lang, native or name))
    return SimpleVocabulary(languages)


@provider(IContextSourceBinder)
def translated_languages(context):
    language_tool = getToolByName(context, "portal_languages")
    language_infos = language_tool.getAvailableLanguages()
    manager = ITranslationManager(context)
    # take care to filter out translated contents
    # wich do no have supported language information
    translated_languages = [
        a for a in manager.get_translated_languages() if a in language_infos
    ]
    content_language = ILanguage(context).get_language()
    if content_language in translated_languages:
        translated_languages.remove(content_language)
    languages = []
    for lang in translated_languages:
        native = language_infos[lang].get("native", None)
        name = language_infos[lang].get("name", lang)
        languages.append(SimpleVocabulary.createTerm(lang, lang, native or name))
    return SimpleVocabulary(languages)


@provider(IContextSourceBinder)
def translated_urls(context):
    manager = ITranslationManager(context)
    translated_languages = manager.get_translated_languages()
    content_language = ILanguage(context).get_language()
    if content_language in translated_languages:
        translated_languages.remove(content_language)
    languages = []
    for lang in translated_languages:
        translation = manager.get_restricted_translation(lang)
        if translation is not None:
            languages.append(
                SimpleVocabulary.createTerm(lang, lang, translation.absolute_url())
            )
    return SimpleVocabulary(languages)


@provider(IContextSourceBinder)
def deletable_languages(context):
    manager = ITranslationManager(context)
    translated_languages = manager.get_translated_languages()
    language_tool = getToolByName(context, "portal_languages")
    language_infos = language_tool.getAvailableLanguages()
    content_language = ILanguage(context).get_language()
    languages = []
    for lang in translated_languages:
        if lang not in content_language:
            native = language_infos[lang].get("native", None)
            name = language_infos[lang].get("name", lang)
            languages.append(SimpleVocabulary.createTerm(lang, lang, native or name))
    return SimpleVocabulary(languages)


def sort_key(language):
    return language[1]


@implementer(IVocabularyFactory)
class AllContentLanguageVocabulary:
    """Vocabulary factory for all content languages in the portal."""

    def __call__(self, context):
        context = getattr(context, "context", context)
        ltool = getToolByName(context, "portal_languages")
        gsm = getGlobalSiteManager()
        util = gsm.queryUtility(ILanguageAvailability)
        if ltool.use_combined_language_codes:
            languages = util.getLanguages(combined=True)
        else:
            languages = util.getLanguages()

        items = [
            (l, languages[l].get("native", languages[l].get("name", l)))
            for l in languages
        ]
        items.sort(key=sort_key)
        items = [SimpleTerm(i[0], i[0], i[1]) for i in items]
        return SimpleVocabulary(items)


AllContentLanguageVocabularyFactory = AllContentLanguageVocabulary()


@implementer(IVocabularyFactory)
class AllAvailableLanguageVocabulary:
    """Vocabulary factory for all enabled languages in the portal."""

    def __call__(self, context):
        context = getattr(context, "context", context)
        ltool = getToolByName(context, "portal_languages")
        gsm = getGlobalSiteManager()
        util = gsm.queryUtility(ILanguageAvailability)
        if ltool.use_combined_language_codes:
            languages = util.getLanguages(combined=True)
        else:
            languages = util.getLanguages()

        supported_languages = ltool.supported_langs
        items = [
            (l, languages[l].get("native", languages[l].get("name", l)))
            for l in languages
            if l in supported_languages
        ]

        items.sort(key=sort_key)
        items = [SimpleTerm(i[0], i[0], i[1]) for i in items]
        return SimpleVocabulary(items)


AllAvailableLanguageVocabularyFactory = AllAvailableLanguageVocabulary()


class RootCatalogVocabularyFactory(CatalogVocabularyFactory):
    """Catalog Vocabulary which always uses the site root"""

    def __call__(self, context, query=None):
        portal = getToolByName(context, "portal_url").getPortalObject()
        return super().__call__(portal, query)



from plone.app.multilingual import _
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.i18n.interfaces import ILanguageSchema
from plone.supermodel import model
from zope import schema
from zope.interface import Attribute
from zope.interface import Interface
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


# CONSTANTS
SHARED_NAME = "shared"

LANGUAGE_INDEPENDENT = ""

ATTRIBUTE_NAME = "_plone.tg"

NOTG = "notg"


class ILanguageRootFolder(Interface):
    """Language Root Folder content type interface"""


class ILanguageIndependentFolder(Interface):
    """Language Independent Folder for content shared between languages"""


# Marker interface
class ITranslatable(Interface):
    """Marker interface for content types that support translation"""


# adapters
class ITranslationFactory(Interface):
    """Adapts ITranslated and is capable of returning
    a translation clone to be added.
    """

    def __call__(language):
        """Create a clone of the context
        for translation to the given language
        """


class ITranslationLocator(Interface):
    """Find a parent folder for a translation.
    Adapts ITranslated.
    """

    def __call__(language):
        """Return a parent folder into which a new translation can be added"""


class ITranslationIdChooser(Interface):
    """Find a valid id for a translation
    Adapts ITranslated.
    """

    def __call__(parent, language):
        """Return a valid id for the translation"""


class ITranslationCloner(Interface):
    """Subscription adapters to perform various aspects of cloning an object.
    Allows componentisation of things like workflow history cloning.
    Adapts ITranslated.
    """

    def __call__(object):
        """Update the translation copy that is being constructed"""


class ITranslationManager(Interface):
    def add_translation(object, intid):
        """
        create the translated content and register the translation
        """

    def remove_translation(language):
        """
        remove translation if exists (unregister the translation)
        """

    def get_translation(language):
        """
        get translation (translated object) if exists
        """

    def get_restricted_translation(language):
        """
        get translation (translated object) if exists and permitted
        """

    def get_translations():
        """
        get all the translated objects (including the context)
        """

    def get_restricted_translations():
        """
        get all the translated objects (including the context) if permitted
        """

    def get_translated_languages():
        """
        get a list of the translated languages
        (language-code like 'en', 'it' etc. )
        """

    def register_translation(language, content):
        """
        register an existing content as translation
        for context
        """

    def update():
        """
        update the item registered in the canonical
        check that there aren't two translations on the same language
        (used for changing the contexts language)
        """

    def query_canonical():
        """
        query if there is an canonical for the context
        used for migration
        """


class ILanguageIndependentFieldsManager(Interface):
    context = Attribute("context", "A translatable object")

    def copy_fields(translation):
        """Copy language independent fields to translation."""


class IMutableTG(Interface):
    """Adapt an object to this interface to manage the TG of an object

    Be sure of what you are doing. Translation Group (TG) is supposed to
    be stable and widely used
    """

    def get():
        """Return the TG of the context"""

    def set(tg):
        """Set the unique id of the context with the tg value."""


class ITG(Interface):
    """Abstract representation of a Translation Group (TG).

    Adapt an object to this interface to obtain its UUID. Adaptation will
    fail if the object does not have a TG (yet).
    """


class IPloneAppMultilingualInstalled(IPloneFormLayer):
    """layer inherits from PloneFormLayer for better LIF widget overriding"""


selector_policies = SimpleVocabulary(
    [
        SimpleTerm(
            value="closest",
            title=_("Search for closest translation in parent's content " "chain."),
        ),
        SimpleTerm(
            value="dialog",
            title=_(
                "Show user dialog with information about the " "available translations."
            ),
        ),
    ]
)


class IMultiLanguageExtraOptionsSchema(ILanguageSchema):
    """Interface for language extra options - control panel fieldset"""

    model.fieldset(
        "multilingual",
        label=_("Multilingual", default="Multilingual"),
        fields=[
            "filter_content",
            "redirect_babel_view",
            "bypass_languageindependent_field_permission_check",
            "buttons_babel_view_up_to_nr_translations",
            "google_translation_key",
            "selector_lookup_translations_policy",
        ],
    )

    filter_content = schema.Bool(
        title=_("heading_filter_content", default="Filter content by language."),
        description=_(
            "description_filter_content",
            default="Filter using language the content on folder_contents",
        ),
        default=True,
        required=False,
    )

    redirect_babel_view = schema.Bool(
        title=_(
            "heading_redirect_babel_view", default="Redirect on creation to babel view."
        ),
        description=_(
            "description_redirect_babel_view",
            default=("After creating a new translation redirecto to babel " "view"),
        ),
        default=True,
        required=False,
    )

    bypass_languageindependent_field_permission_check = schema.Bool(
        title=_(
            "heading_bypass_languageindependent_field_permission_check",
            default="Bypass language independent field permission check",
        ),
        description=_(
            "description_bypass_languageindependent_field_permission_check",
            default="When updating an object with language independent the "
            "field will be synced to all target languages. That can "
            "produce Unauthorized error messages because if the "
            "editor of the canonical object is not allowed to update "
            "the target language objects. Enabling this bypasses "
            "this permission check. This could also be dangerous, so "
            "think about possible security issues before enabling "
            "this.",
        ),
        default=False,
        required=False,
    )

    buttons_babel_view_up_to_nr_translations = schema.Int(
        title=_(
            "heading_buttons_babel_view_up_to_nr_translations",
            default="Use buttons in the bable view for up to how many " "translations?",
        ),
        description=_(
            "description_buttons_babel_view_up_to_nr_translations",
            default="When there are many translations for an item, the "
            "number of displayed buttons for them might get too "
            "large to fit inside the template. Choose here from "
            "which number onwards a drop-down menu will be displayed "
            "instead of buttons. Zero means that buttons will always "
            "be used.",
        ),
        default=7,
        required=False,
    )

    google_translation_key = schema.TextLine(
        title=_("heading_google_translation_key", default="Google Translation API Key"),
        description=_(
            "description_google_translation_key",
            default="Is a paying API in order to use google translation " "service",
        ),
        required=False,
    )

    selector_lookup_translations_policy = schema.Choice(
        title=_(
            "heading_selector_lookup_translations_policy",
            default="The policy used to determine how the lookup for "
            "available translations will be made by the language "
            "selector.",
        ),
        description=_(
            "description_selector_lookup_translations_policy",
            default="The default language used for the content "
            "and the UI of this site.",
        ),
        required=True,
        vocabulary=selector_policies,
    )



from logging import getLogger
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.multilingual.itg import addAttributeTG
from plone.base.interfaces import INonInstallable
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):
        """Prevents uninstall profile from showing up in the profile list
        when creating a Plone site.

        """
        return [
            "plone.app.multilingual:uninstall",
        ]


def init_pam(tool):
    """After installation run setup to create LRF and LIF."""
    setup_tool = SetupMultilingualSite()
    setup_tool.setupSite(getSite())


def step_default_various(context):
    if context.readDataFile("plone.app.multilingual_default.txt") is None:
        return
    portal = context.getSite()
    enable_translatable_behavior(portal)

    # Add the attribute to the site root now so plone.protect won't cry.
    addAttributeTG(portal, None)


def enable_translatable_behavior(portal):
    types_tool = portal.portal_types

    # Iterate through all Dexterity content type, except the site root
    all_ftis = types_tool.listTypeInfo()
    dx_ftis = (
        fti
        for fti in all_ftis
        if getattr(fti, "behaviors", False) and fti.getId() != "Plone Site"
    )
    for fti in dx_ftis:
        # Enable translatable behavior for all types
        if "plone.translatable" not in fti.behaviors:
            behaviors = list(fti.behaviors)
            behaviors.append("plone.translatable")
            fti._updateProperty("behaviors", tuple(behaviors))


def step_uninstall_various(context):
    if context.readDataFile("plone.app.multilingual_uninstall.txt") is None:
        return
    portal = context.getSite()
    disable_translatable_behavior(portal)
    disable_language_switcher(portal)


def disable_translatable_behavior(portal):
    """Remove p.a.multilingual behaviors from p.a.contenttypes types"""
    types_tool = portal.portal_types

    # Iterate through all Dexterity content type
    all_ftis = types_tool.listTypeInfo()
    dx_ftis = [x for x in all_ftis if getattr(x, "behaviors", False)]
    for fti in dx_ftis:
        # Disable translatable behavior from all types
        behaviors = [i for i in fti.behaviors if i != "plone.translatable"]
        fti._updateProperty("behaviors", behaviors)


def disable_language_switcher(portal):
    """Remove the use of language-switcher as default view for Plone Site"""
    tt = getToolByName(portal, "portal_types")
    site = tt["Plone Site"]
    site.view_methods = tuple(
        method for method in site.view_methods if method != "language-switcher"
    )
    if site.default_view == "language-switcher":
        site.default_view = "listing_view"

    log = getLogger("setuphandlers.disable_language_switcher")
    log.info("Language switcher disabled")



from .interfaces import ILanguageIndependentField
from plone.app.event.dx.behaviors import IEventBasic
from zope.interface import alsoProvides

# configure existing behaviors to be language independent
alsoProvides(IEventBasic["start"], ILanguageIndependentField)
alsoProvides(IEventBasic["end"], ILanguageIndependentField)
alsoProvides(IEventBasic["whole_day"], ILanguageIndependentField)
alsoProvides(IEventBasic["open_end"], ILanguageIndependentField)
